/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "timeVaryingNonuniformFixedValueFvPatchField.H"
#include "graph.H"
#include "IFstream.H"
#include "interpolateXY.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class Type>
timeVaryingNonuniformFixedValueFvPatchField<Type>::timeVaryingNonuniformFixedValueFvPatchField
(
    const fvPatch& p,
    const Field<Type>& iF
)
:
    fixedValueFvPatchField<Type>(p, iF),
    timeData_()
{}


template<class Type>
timeVaryingNonuniformFixedValueFvPatchField<Type>::timeVaryingNonuniformFixedValueFvPatchField
(
    const timeVaryingNonuniformFixedValueFvPatchField<Type>& ptf,
    const fvPatch& p,
    const Field<Type>& iF,
    const fvPatchFieldMapper&
)
:
    fixedValueFvPatchField<Type>(p, iF),
    timeData_(ptf.timeData_)
{}


template<class Type>
timeVaryingNonuniformFixedValueFvPatchField<Type>::timeVaryingNonuniformFixedValueFvPatchField
(
    const fvPatch& p,
    const Field<Type>& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchField<Type>(p, iF),
    timeData_(this->db().time(), this->patch().name())
{
    if (dict.found("value"))
    {
        fvPatchField<Type>::operator==(Field<Type>("value", dict, p.size()));
    }
    else
    {
        updateCoeffs();
    }
}


template<class Type>
timeVaryingNonuniformFixedValueFvPatchField<Type>::timeVaryingNonuniformFixedValueFvPatchField
(
    const timeVaryingNonuniformFixedValueFvPatchField<Type>& ptf,
    const Field<Type>& iF
)
:
    fixedValueFvPatchField<Type>(ptf, iF),
    timeData_(ptf.timeData_)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

// Update the coefficients associated with the patch field
template<class Type>
void timeVaryingNonuniformFixedValueFvPatchField<Type>::updateCoeffs()
{
    if (this->updated())
    {
        return;
    }

    if (this->size())
    {

        scalar t=this->db().time().value();
        label I=0;

        if (t<this->timeData_[0].time_)
        {
            WarningIn
                (
                    "timeVaryingNonuniformFixedValueFvPatchField<Type>::updateCoeffs()"
                )   << "current time (" << t
                    << ") is less than the minimum in the data table ("
                    << this->timeData_[0].time_ << ')' << endl
                    << "    Continuing with the value for the smallest time"
                    << endl;
            I=1;
        }

        else

            if (t>this->timeData_[this->timeData_.size()-1].time_)
            {
                WarningIn
                    (
                        "timeVaryingNonuniformFixedValueFvPatchField<Type>::updateCoeffs()"
                    )   << "current time (" << t
                        << ") is greater than the maximum in the data table ("
                        << this->timeData_[this->timeData_.size()-1].time_ << ')' << endl
                        << "    Continuing with the value for the largest time"
                        << endl;

                I=this->timeData_.size()-1;
                t=this->timeData_[this->timeData_.size()-1].time_;
            }

            else

            {
                for (I=1;I<this->timeData_.size()-1;I++)
                    if (this->timeData_[I].time_>t) break;
            }

        for (int J=0;J<2;J++)
        {
            if (!(cache_[J]==this->timeData_[I+J-1]) || cache_[J].isInvalid())
            {
                cache_[J]=this->timeData_[I+J-1];
                //Info<<"create J:"<<J<<endl;
                IFstream fI(this->timeData_[I+J-1].fileName_);
                autoPtr<List<Type> > dataFromFile(new List<Type>(fI));
                if (dataFromFile().size() == this->size())
                    cache_[J]=dataFromFile;
                else
                {
                    FatalErrorIn
                        (
                            "timeVaryingNonuniformFixedValueFvPatchField<Type>::updateCoeffs()"
                        )   
                        << "List size in file " 
                            << this->timeData_[I+J-1].fileName_ 
                            << " (" << dataFromFile().size() << ")"
                            << " does not match size of current patch ("
                            << this->size() << ")." << endl;
                }
            }
        }

        fvPatchField<Type>::operator=
            (
                (
                    cache_[1].fieldPtr_() * (t-this->timeData_[I-1].time_)
                    + 
                    cache_[0].fieldPtr_() * (this->timeData_[I].time_-t)
                )
                /
                (this->timeData_[I].time_ - this->timeData_[I-1].time_)
            );
    }
    fixedValueFvPatchField<Type>::updateCoeffs();
}


// Write
template<class Type>
void timeVaryingNonuniformFixedValueFvPatchField<Type>::write(Ostream& os) const
{
    fvPatchField<Type>::write(os);
    this->writeEntry("value", os);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
